<?php

use UnicaenCode\Form\ElementMaker;
use UnicaenCode\Util;

/**
 * @var $this       \Application\View\Renderer\PhpRenderer
 * @var $controller \Zend\Mvc\Controller\AbstractController
 * @var $container  \Interop\Container\ContainerInterface
 * @var $viewName   string
 * @var $viewFile   string
 */

?>
    <h1>Génération d'une Classmap pour optimiser l'autoload</h1>
<?php

$form = new \Zend\Form\Form();
$form->add(ElementMaker::selectModule(
    'module', 'Module concerné'
));
$form->add(ElementMaker::submit('generate', 'Générer la Classmap'));
$form->setData($controller->getRequest()->getPost());

Util::displayForm($form);

if ($controller->getRequest()->isPost() && $form->isValid()) {
    affModMap($form->get('module')->getValue());
}


function affModMap(string $moduleName)
{
    $module = Util::introspection()->getModule($moduleName);

    $sCodeGenerator = Util::codeGenerator();

    $params = [
        'template'     => 'autoload_classmap',
        'filename'     => 'module/' . $module['relPath'] . '/autoload_classmap.php',
        'map'          => genModMap($moduleName),
        'echoExpanded' => true,
    ];

    ?>
    <div class="alert alert-info">Le fichier est récupérable dans le
        dossier <?php echo $sCodeGenerator->getOutputDir() ?></div>

    <h3>Etape 3 : Déclaration dans le fichier de configuration</h3>
    <?php $sCodeGenerator->generate($params); ?>
    <div class="alert alert-warning">
        Vous devez vous-même placer ces informations dans le fichier de configuration de votre
        module.
    </div>

    <?php
}



function genModMap(string $moduleName)
{
    $sIntrospection = Util::introspection();

    $module = $sIntrospection->getModule($moduleName);
    if (!isset($module)) {
        throw new \Exception('Le module "' . $moduleName . '" est introuvable');
    }

    $map = $sIntrospection->getClasses($module['namespace']);

    $mc = 0;
    foreach ($map as $c => $f) {
        $lc = strlen($c);
        if ($lc > $mc) $mc = $lc;
    }
    $php = '';
    foreach ($map as $c => $f) {
        $php .= "    " . str_pad("'$c'", $mc + 2, ' ') . " => __DIR__ . '$f',\n";
    }

    return trim($php);
}