<?php

namespace UnicaenCode\Generator;

use UnicaenCode\Util;

class AwareTrait implements GeneratorInterface
{
    /**
     * @param array $params
     *
     * Paramètres en entrée :
     *
     * - class => string nom de la classe pour laquelle le AwareTrait sera généré
     * - useGetter => boolean true si on génère un getter, false sinon
     * - subDir => boolean true si on place le trait dans un sous-dossier Traits
     *
     * @return array
     */
    public function generate(array $params): array
    {
        $class     = $params['class'] ?? null;
        $template  = $params['template'] ?? 'AwareTrait';
        $useGetter = (bool)($params['useGetter'] ?? true);
        $subDir    = (bool)($params['subDir'] ?? true);
        $type      = $params['type'] ?? Util::classType($class);

        if (!$type && false !== strpos($class, 'Entity')) {
            $type = 'Entity';
        }

        $namespace = Util::classNamespace($class) . ($subDir ? '\\Traits' : '');

        $targetClass     = $class;
        $targetClassname = Util::classClassname($class);

        $classname = $targetClassname . 'AwareTrait';

        $module = Util::classModule($class);

        if ('Entity' == $type) {
            $method = Util::classClassname($class);
        } else {
            $method     = Util::classTruncated($class, $module);
            $methodBase = Util::classRoot($method);
            $method     = Util::classCamelCase($method);
            if (substr($method, -strlen($methodBase)) == $methodBase) {
                $method = substr($method, 0, -strlen($methodBase));
            }
        }
        $variable = lcfirst($method);

        $filename = Util::classFilename($namespace . '\\' . $classname);

        $params['template']        = $template;
        $params['filename']        = $filename;
        $params['namespace']       = $namespace;
        $params['subDir']          = $subDir;
        $params['targetClass']     = $targetClass;
        $params['classname']       = $classname;
        $params['targetClassname'] = $targetClassname;
        $params['variable']        = $variable;
        $params['method']          = $method;
        $params['useGetter']       = $useGetter;

        return $params;
    }



    public function generateFromParent(array $parentParams, array $params): array
    {
        $useParams = ['class', 'type'];
        foreach ($useParams as $useParam) {
            if (isset($parentParams[$useParam])) $params[$useParam] = $parentParams[$useParam];
        }

        return $params;
    }

}