<?php

namespace UnicaenCode\Generator;

use UnicaenCode\Util;

class ControllerConfig implements GeneratorInterface
{
    /**
     * @param array $params
     *
     * Paramètres en entrée :
     *
     * - class => string nom de la classe pour laquelle le AwareTrait sera généré
     *
     * @return array
     */
    public function generate(array $params): array
    {
        $class    = $params['class'] ?? null;
        $template = $params['template'] ?? 'ControllerConfig';
        $author   = $params['author'] ?? Util::getAuthor();
        $route    = $params['route'] ?? null;


        $namespace = Util::classModule($class, Util::CLASS_MODULE_NAMESPACE);
        $classname = Util::classClassname($class);

        if (substr($classname, -strlen('Controller')) == 'Controller') {
            $name = substr($class, 0, -strlen('Controller'));

            if (!$route) {
                $route = lcfirst(substr($classname, 0, -strlen('Controller')));
            }
        } else {
            $name = $class;
            if (!$route) {
                $route = lcfirst($classname);
            }
        }

        $filename = Util::classModule($class, Util::CLASS_MODULE_CONFIG)['relPath'] . '/config/module.config.php';


        $params['namespace']    = $namespace;
        $params['classname']    = $classname;
        $params['template']     = $template;
        $params['name']         = $name;
        $params['author']       = $author;
        $params['route']        = $route;
        $params['filename']     = $filename;
        $params['write']        = false; // ne jamais écrire cette config : tout est à copier/coller
        $params['factoryClass'] = $this->makeFactoryClass($params);

        return $params;
    }



    protected function makeFactoryClass(array $params): string
    {
        $factoryParams = [
            'generator' => 'Factory',
            'class'     => $params['class'],
            'subDir'    => (bool)($params['subDir'] ?? true),
            'type'      => 'Controller',
            'author'    => $params['author'] ?? Util::getAuthor(),
        ];
        $factoryParams = Util::codeGenerator()->generate($factoryParams);

        return $factoryParams['namespace'] . '\\' . $factoryParams['classname'];
    }



    public function generateFromParent(array $parentParams, array $params): array
    {
        $useParams = ['class', 'type', 'author'];
        foreach ($useParams as $useParam) {
            if (isset($parentParams[$useParam])) $params[$useParam] = $parentParams[$useParam];
        }

        if (isset($parentParams['factory']['subDir']) && !isset($params['subDir'])) $params['subDir'] = $parentParams['factory']['subDir'];

        return $params;
    }

}